@@ -34,12 +34,12 @@ if (!isGeneric("Schur"))
 if (!isGeneric("unpack"))
     setGeneric("unpack", function(x, ...) standardGeneric("unpack"))
 
-if (!isGeneric("%p%"))
-    setGeneric("%p%", function(a, b) standardGeneric("%p%"))
-
-if (!isGeneric("coef<-"))
-    setGeneric("coef<-", function(object, ..., value)
-               standardGeneric("coef<-"))
+##- if (!isGeneric("%p%"))
+##-     setGeneric("%p%", function(a, b) standardGeneric("%p%"))
+##-
+##- if (!isGeneric("coef<-"))
+##-     setGeneric("coef<-", function(object, ..., value)
+##-                standardGeneric("coef<-"))
 
 
 ## The generics pdFactor, pdMatrix, corFactor and corMatrix will be deprecated
